<?php

require_once 'Vmap.class.inc';
require_once 'Maps.class.inc';
require_once 'Services.class.inc';
require_once 'Layers.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/ws/vitis/Users.class.inc';
require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/class/vitis_lib/Connection.class.inc';

/**
 * \file Map.class.inc
 * \class Map
 *
 * \author Armand Bahi <armand.bahi@veremes.com>.
 *
 * \brief This file contains the Map php class
 *
 * This class defines operation for one Map
 *
 */
class MapCatalog extends Vmap {

    public $oError;

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
        $this->aValues = $aValues;
        $this->aPath = $aPath;
        $this->aProperties = $properties;
        $this->aValues['getGroup'] = true;
        $this->oConnection = new Connection($this->aValues, $this->aProperties);
    }

    /**
     * @SWG\Get(path="/mapcatalog",
     *   tags={"Catalog"},
     *   summary="Get Map",
     *   description="Request to get Map by id",
     *   operationId="GET",
     *   produces={"application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/mapcatalog")
     *     )
     *  )
     */

    /**
     * get informations about Map
     */
    function GET() {
        require $this->sRessourcesFile;

        $aValues = $this->aValues;
        $aPath = $this->aPath;
        $properties = $this->aProperties;
        $oConnection = $this->oConnection;

        // Récupère les groupes correspondants
        $sGroups = $this->oConnection->sesGroup;

        // Récupère les cartes correspondante
        $aMaps = $this->getMaps($sGroups);

        // Récupère les services correspondants
        $aServices = $this->getServices();

        $MapCatalog['usedMap'] = $this->getDefaultMap($aMaps);
        $MapCatalog['defaultExtent'] = $this->getDefaultExtent();
        $MapCatalog['maps'] = $aMaps;
        $MapCatalog['services'] = $aServices;

        $this->aFields = $MapCatalog;

        $aXmlRacineAttribute['status'] = 1;
        $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);

        // Génération du flux wms privé.
        if (!empty($properties["use_veremap_api"]) && $properties["use_veremap_api"] === true) {
            // 1- Récupère le token veremap 2015
            $params = array(
                "user" => $oConnection->oBd->login,
                "password" => $oConnection->oBd->mdp,
                "duration" => 600
            );
            $oVeremap = json_decode($this->httpPost($properties["api_veremap"] . "/vitis/privatetoken", $params));
            if (is_object($oVeremap)) {
                $veremapToken = $oVeremap->token;
                // 2- Génère le flux privé qui aura pour nom de mapFile le token de vitis 2016
                $params = array(
                    "token" => $veremapToken,
                    "fileName" => hash('sha256', $aValues['token'])
                );
                $this->httpPost($properties["api_veremap"] . "/veremap/WMS", $params);
            }
        }
        // Création du fichier ".map" du flux wms privé de Vmap4MapServer (nom du fichier = token de l'utilisateur connecté).
        if (!empty($properties["use_vm4ms_api"]) && $properties["use_vm4ms_api"] === true && file_exists(dirname($_SERVER['SCRIPT_FILENAME']) . '/ws/vm4ms/WmsServices.class.inc')) {
            require_once dirname($_SERVER['SCRIPT_FILENAME']) . '/ws/vm4ms/WmsServices.class.inc';

            $aPath = array('vm4ms', 'wmsservices', 'private', 'MapFile');

            $aValues = array(
                'token' => $this->aValues['token'],
                'my_vitis_id' => $properties['private_wms_service'],
                'creation' => 'true',
                'type' => 'prod'
            );

            $oWmsServices = new WmsServices($aPath, $aValues, $this->aProperties, $this->oConnection);
            $oWmsServices->GET();
            // $oWmsServices->createMapFile();
        }
        return $sMessage;
    }

    /**
     * Get the maps for the group_id
     * @param string $sGroups
     * @return array
     */
    function getMaps($sGroups) {

        if (empty($sGroups)) {
            return array();
        }

        $sGroups = str_replace(',', '|', $sGroups);

        $aValues = $this->aValues;
        $aPath = $this->aPath;
        $properties = $this->aProperties;
        $oConnection = $this->oConnection;
        $filterList = '';

        $aSQLParams = array(
            'sSchema' => array('value' => $this->aProperties['schema_vmap'], 'type' => 'column_name'),
            'sGroups' => array('value' => $sGroups, 'type' => 'group')
        );
        $sSql = "SELECT map_id, group_id FROM [sSchema].map_group WHERE \"group_id\" in ([sGroups])";

        $oResult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
        if (gettype($oResult) == 'object') {
            $aLayers_infos = Array();
            while ($aLigne = $this->oConnection->oBd->ligneSuivante($oResult)) {
                // stocke le layer id dans la chaine de caractères
                if ($filterList == '')
                    $filterList .= $aLigne['map_id'];
                else
                    $filterList .= ',' . $aLigne['map_id'];
            }
        } else {
            return Array();
        }

        unset($aValues['my_vitis_id']);
        $aValues['filter'] = '{"column":"map_id","compare_operator":"IN","value":[' . $filterList . ']}';
        $aValues['attributs'] = 'name|description|theme_name|theme_description|extent_crs_id|map_id|thumbnail|catalog_index';
        $aValues['order_by'] = 'catalog_index';

        $oMaps = new Maps(Array('vmap', 'map'), $aValues, $this->aProperties);
        $oMaps->GET();

        // Transforme $oMaps en tableau de Map
        $aMaps = Array();
        foreach ($oMaps->aObjects as $oMap) {
            $oMap->aFields['url'] = $properties['web_server_name'] . '/' . $properties['services_alias'] . '/vmap/mapjsons/' . $oMap->aFields['map_id'];
            array_push($aMaps, $oMap->aFields);
        }

        return $aMaps;
    }

    /**
     * Get the services avaliables (with the layers params for bing and ign)
     * @return array
     */
    function getServices() {

        $aValues = $this->aValues;
        $aPath = $this->aPath;
        $properties = $this->aProperties;
        $oConnection = $this->oConnection;

        unset($aValues['my_vitis_id']);
        $oServices = new Services(Array('vmap', 'services'), $aValues, $this->aProperties);
        $oServices->GET();

        $aServices['wms'] = Array();
        $aServices['osm'] = Array();
        $aServices['bing'] = Array();
        $aServices['ign'] = Array();
        $aServices['wmts'] = Array();
        $aServices['xyz'] = Array();
        foreach ($oServices->aObjects as $oService) {
            switch ($oService->aFields['service_type_id']) {
                case 'tilewms':
                case 'imagewms':
                    $oService->aFields['url'] = str_replace('[token]', hash('sha256', $aValues['token']), $oService->aFields['url']);
                    if (isset($this->aProperties['ms_cgi_url'])) {
                        $oService->aFields['url'] = str_replace('[ms_cgi_url]', $this->aProperties['ms_cgi_url'], $oService->aFields['url']);
                    }
                    array_push($aServices['wms'], $oService->aFields);
                    break;
                case 'wmts':
                    array_push($aServices['wmts'], $oService->aFields);
                    break;
                case 'xyz':
                    array_push($aServices['xyz'], $oService->aFields);
                    break;
                case 'osm':
                    array_push($aServices['osm'], $oService->aFields);
                    break;
                case 'bing':
                    $oService->aFields['imagerySet'] = $oService->aFields['imagery'];
                    $oService->aFields['culture'] = $oService->aFields['lang'];
                    array_push($aServices['bing'], $oService->aFields);
                    break;
                case 'geoportail':
                    break;
            }
        }

        return $aServices;
    }

    /**
     * getDefaultMap - get the default map index
     *
     * @param  {array} $aMaps
     * @return {number} default map index
     */
    function getDefaultMap($aMaps) {
        require $this->sRessourcesFile;

        $iDefaultMapId = false;
        $iDefaultMapIndex = 0;

        // Récupère la carte par défaut de l'utilisateur en cours
        $sSql = $aSql['getCurrentUserDefaultMap'];
        $aSQLParams = array(
            'sSchemaFramework' => array('value' => $this->aProperties['schema_framework'], 'type' => 'schema_name'),
            'sSchemaVmap' => array('value' => $this->aProperties['schema_vmap'], 'type' => 'column_name')
        );
        $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
        while($aLigne=$this->oConnection->oBd->ligneSuivante ($oPDOresult)) {
            $iDefaultMapId = $aLigne['map_id'];
        }

        if ($iDefaultMapId !== false) {
            for ($i=0; $i < count($aMaps); $i++) {
                if ($aMaps[$i]['map_id'] == $iDefaultMapId) {
                    $iDefaultMapIndex = $i;
                }
            }
        }

        return $iDefaultMapIndex;
    }

    /**
     * getDefaultExtent - get the default extent
     *
     * @return {number} default extent
     */
    function getDefaultExtent() {
        require $this->sRessourcesFile;

        $sUserDefaultExtent = null;

        // Récupère l'étendue par défaut de l'utilisateur en cours
        $sSql = $aSql['getCurrentUserDefaultMap'];
        $aSQLParams = array(
            'sSchemaFramework' => array('value' => $this->aProperties['schema_framework'], 'type' => 'schema_name'),
            'sSchemaVmap' => array('value' => $this->aProperties['schema_vmap'], 'type' => 'column_name')
        );
        $oPDOresult = $this->oConnection->oBd->executeWithParams($sSql, $aSQLParams);
        while($aLigne=$this->oConnection->oBd->ligneSuivante ($oPDOresult)) {
            $sUserDefaultExtent = $aLigne['extent'];
        }

        return $sUserDefaultExtent;
    }

    function httpPost($url, $params) {
        $postData = '';
        //create name value pairs seperated by &
        foreach ($params as $k => $v) {
            $postData .= $k . '=' . $v . '&';
        }
        $postData = rtrim($postData, '&');

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_POST, count($postData));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);

        $output = curl_exec($ch);

        curl_close($ch);
        return $output;
    }

}

?>
